# Mockingbird CocoaPods Example

A minimal example of [Mockingbird](https://github.com/birdrides/mockingbird) integrated into an Xcode project using
CocoaPods.

Having issues setting up Mockingbird? Join the [Slack channel](https://slofile.com/slack/birdopensource) or
[file an issue](https://github.com/birdrides/mockingbird/issues/new/choose).

## Tutorial

### Configure CocoaPods

Create the Podfile in the Xcode project root directory.

```bash
$ cd iOSMockingbirdExample-CocoaPods
$ touch Podfile
```

Add Mockingbird as a dependency to the test target in the Podfile.

```ruby
target 'iOSMockingbirdExample-CocoaPodsTests' do
  use_frameworks!
  pod 'MockingbirdFramework', '~> 0.12'
end
```

### Install Mockingbird

Initialize the pod and install the CLI.

```bash
$ pod install
$ (cd Pods/MockingbirdFramework && make install-prebuilt)
```

Then configure the test target.

```bash
$ mockingbird install \
  --target iOSMockingbirdExample-CocoaPodsTests \
  --source iOSMockingbirdExample-CocoaPods
```

Finally, download the starter supporting source files into your project root.

```bash
$ mockingbird download starter-pack
```

### Run Tests

Open the Xcode workspace generated by CocoaPods.

```bash
$ open iOSMockingbirdExample-CocoaPods.xcworkspace
```

Take a peek at the example test and sources and then run the tests (⌘+U).

- [`TreeTests.swift`](iOSMockingbirdExample-CocoaPodsTests/TreeTests.swift)
- [`Tree.swift`](iOSMockingbirdExample-CocoaPods/Tree.swift)
- [`Bird.swift`](iOSMockingbirdExample-CocoaPods/Bird.swift)

Bonus: 
- [`.mockingbird-ignore`](iOSMockingbirdExample-CocoaPods/.mockingbird-ignore)
- [`.gitignore`](.gitignore)
